# window.py
#
# Copyright 2024 Isaac Beverly
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# SPDX-License-Identifier: GPL-2.0-or-later

import datetime
import json
import logging
from multiprocessing.pool import ThreadPool
import os
from subprocess import CompletedProcess
import time
from gi.repository import Adw, Gio, Gtk

from .bootc import get_state, get_available_update, start_upgrade


logger = logging.getLogger(__name__)


@Gtk.Template(resource_path='/org/heliumos/bootc_gtk/window.ui')
class BootcGtkWindow(Adw.ApplicationWindow):
    __gtype_name__ = 'BootcGtkWindow'

    header_bar = Gtk.Template.Child()
    refresh_button = Gtk.Template.Child()
    stack = Gtk.Template.Child()
    overview_page = Gtk.Template.Child()
    overview_status = Gtk.Template.Child()
    upgrade_page = Gtk.Template.Child()
    upgrade_status = Gtk.Template.Child()
    switcher_bar = Gtk.Template.Child()
    upgrade_stack = Gtk.Template.Child()
    check_for_update_button = Gtk.Template.Child()
    upgrade_button = Gtk.Template.Child()
    upgrade_spinner = Gtk.Template.Child()
    upgrade_complete_image = Gtk.Template.Child()
    upgrade_error_image = Gtk.Template.Child()


    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        window_title = os.getenv("BOOTC_GTK_TITLE", "System Images")
        self.set_title(window_title)

        self.current_image: Optional[Image] = None
        self.upgrade_image: Optional[Image] = None

        self.pool = ThreadPool()

        refresh_action = Gio.SimpleAction(name="refresh")
        refresh_action.connect("activate", self.refresh)
        self.add_action(refresh_action)

        check_for_update_action = Gio.SimpleAction(name="check_for_update")
        check_for_update_action.connect("activate", self.check_for_update)
        self.add_action(check_for_update_action)

        upgrade_action = Gio.SimpleAction(name="upgrade")
        upgrade_action.connect("activate", self.upgrade)
        self.add_action(upgrade_action)

        self.refresh()


    def refresh(self, *args, **kwargs):
        logger.info("Refreshing")
        state = get_state()
        logger.debug(f"{state=}")

        if state is None:
            self.overview_status.set_title("Unable to check status")
            return

        if state.is_rollback_queued:
            self.overview_status.set_title("Reboot to complete rollback")
            return

        if not state.staged is None:
            self.overview_status.set_title("Reboot to complete upgrade")
            return

        self.overview_status.set_title("No changes queued")
        self.current_image = state.booted


    def check_for_update(self, *args, **kwargs):
        logger.info("Checking for update")
        available_update = get_available_update()
        logger.debug(f"{available_update=}")

        if isinstance(available_update, Exception):
            self.upgrade_status.set_title("Unable to check for update")
            logger.error(available_update)
            logger.warning("Unable to check for update")
            return

        if available_update is None:
            self.upgrade_status.set_title("Up to date")
            return

        self.upgrade_image = available_update
        self.upgrade_status.set_title("Upgrade available")
        self.upgrade_status.set_description(
            f"{available_update.image}\n{available_update.digest}"
        )
        self.upgrade_stack.set_visible_child(self.upgrade_button)


    def upgrade(self, *args, **kwargs):
        logger.info("Upgrading")
        self.upgrade_status.set_title("Upgrading system")
        if self.current_image:
            self.upgrade_status.set_description(
                f"Booted image:   {self.current_image.digest[0:14]}\nNew image:         {self.upgrade_image.digest[0:14]}"
            )
        else:
            self.upgrade_status.set_description(
                f"New image:         {self.upgrade_image.digest[0:14]}"
            )

        self.upgrade_stack.set_visible_child(self.upgrade_spinner)

        self.pool.apply_async(start_upgrade, (self.post_upgrade,))


    def post_upgrade(self, proc: CompletedProcess):
        logger.debug("Post upgrade")

        if proc.returncode == 0:
            self.upgrade_status.set_title("Upgrade complete!")
            self.upgrade_status.set_description(f"Upgraded to image {self.upgrade_image.digest[0:14]}")
            self.upgrade_stack.set_visible_child(self.upgrade_complete_image)
            self.overview_status.set_title("Reboot to complete upgrade")
        else:
            logger.error(proc.stderr)
            self.upgrade_status.set_title("Error!")
            self.upgrade_status.set_description(f"An error occurred:\n{proc.stderr}")
            self.upgrade_stack.set_visible_child(self.upgrade_error_image)

